/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * Endian.hpp
 *
 *  Created on: 2011/02/28
 *      Author: HARK Development Team
 */

#ifndef ENDIAN_HPP_
#define ENDIAN_HPP_

#include <stdint.h>

namespace
{
const uint16_t endianInt = 1;
const bool environmentIsLittle = *reinterpret_cast<const uint8_t*>(&endianInt) == 1;
}

template <typename T>
T InterpretBytes(const char* bytes, bool little)
{
  if (little == environmentIsLittle)
  {
    return *reinterpret_cast<const T*>(bytes);
  }
  else
  {
    char mirror[sizeof(T)];
    for (size_t i = 0; i < sizeof(T); ++i)
    {
      mirror[i] = bytes[sizeof(T) - 1 - i];
    }
    return *reinterpret_cast<const T*>(mirror);
  }
}

template <typename T>
inline void InterpretBytes(T& result, const char* bytes, bool little)
{
  result = InterpretBytes<T>(bytes, little);
}

#endif /* ENDIAN_HPP_ */
